#include <u.h>
#include <libc.h>
#include <utime.h>

typedef struct Idtab Idtab;

extern Idtab *usertab;
extern Idtab *grouptab;

extern long	altgetnumid(Idtab*, char*);
extern char*	altgetstrid(Idtab*, long);
extern char*	altnouserstr(long);
extern char*	altnogroupstr(long);
extern char*	estrdup(char*);

static void	stat2dir(Dir*, struct stat*);
static void	cleandir(Dir*);

Dir*
dirfstat(int fd)
{
	Dir *d;
	struct stat sbuf;

	if(fstat(fd, &sbuf) < 0)
		return nil;
	d = malloc(sizeof *d);
	memset(d, 0, sizeof *d);
	stat2dir(d, &sbuf);
	return d;
}

/* Botch: do we want lstat(2) or stat(2) here? */

Dir*
dirstat(char *f)
{
	Dir *d;
	struct stat sbuf;
	char buf[256+1];

	if(stat(f, &sbuf) < 0 && lstat(f, &sbuf) < 0)
		return nil;
	d = malloc(sizeof *d);
	if(d == nil)
		return nil;
	memset(d, 0, sizeof *d);
	stat2dir(d, &sbuf);
	d->name = estrdup(f);        /* stat2dir calls cleandir */
	if(lstat(f, &sbuf) < 0){
		freedir(d);
		return nil;
	}
	if(S_ISLNK(sbuf.st_mode)){
		d->mode |= DMLINK;
		d->mode &= ~DMDIR;
		memset(buf, 0, sizeof buf);
		if(readlink(f, buf, sizeof buf) < 0){
			freedir(d);
			return nil;
		}
		d->symlink = estrdup(buf);  /* stat2dir calls cleandir */
	}

	return d;
}

/* XXX: would be better to allocate everything in a single block */
void
freedir(Dir *d)
{
	if(d == nil)
		return;
	cleandir(d);
	free(d);
	return;
}

int
dirfwstat(int fd, Dir *d)
{
	/* Botch: no (portable) way to set times (?) */
	fchown(fd, d->uidno, d->gidno);
	fchmod(fd, d->mode);

	return -1;
}

int
dirwstat(char *f, Dir *d)
{
	struct utimbuf t;

	memset(&t, 0, sizeof(struct utimbuf));

	/* Botch: lchown(2) */
	chown(f, d->uidno, d->gidno);
	chmod(f, d->mode);

	t.actime = d->atime;
	t.modtime = d->mtime;
	utime(f, &t);

	/* Botch: size, owner, group, -rename-, etc. */

	return 0;
}

static void
stat2dir(Dir *dir, struct stat *s)
{
	struct passwd *p;
	struct group *g;
	ulong q;

	cleandir(dir);
	memset(dir, 0, sizeof(dir));

	if((dir->uid = altgetstrid(usertab, s->st_uid)) == nil){
		if((p = getpwuid(s->st_uid)) != nil)
			dir->uid = estrdup(p->pw_name);
		else
			dir->uid = altnouserstr(s->st_uid);
	}
	if((dir->gid = altgetstrid(grouptab, s->st_gid)) == nil){
		if((g = getgrgid(s->st_gid)) != nil)
			dir->gid = estrdup(g->gr_name);
		else
			dir->gid = altnogroupstr(s->st_gid);
	}
	q = 0;
	if(S_ISDIR(s->st_mode))
		q |= DMDIR;
	q |= s->st_ino & 0x00FFFFFFUL;
	dir->qid.path = q;
	dir->qid.vers = s->st_mtime;
	dir->mode = (dir->qid.path&DMDIR)|(s->st_mode&0777);
	if(S_ISLNK(s->st_mode))
		dir->mode |= DMLINK;
	dir->atime = s->st_atime;
	dir->mtime = s->st_mtime;
	dir->length = s->st_size;
	dir->dev = s->st_dev;
	dir->type = 'M';
	if(S_ISFIFO(s->st_mode))
		dir->type = '|';

	/* Unix port-specific garbage */
	dir->ctime = s->st_ctime;
	dir->uidno = s->st_uid;
	dir->gidno = s->st_gid;
	dir->unixmode = s->st_mode;
}

static void
cleandir(Dir *dir)
{
	if(dir->name)
		free(dir->name);
	if(dir->uid)
		free(dir->uid);
	if(dir->gid)
		free(dir->gid);
	if(dir->muid)
		free(dir->muid);
	if(dir->symlink)
		free(dir->symlink);
}
