#include <u.h>
#include <libc.h>
#include <bio.h>
#include <libsec.h>
#include <ctype.h>

#include "iso9660.h"

#include <grp.h>
#include <pwd.h>

static long	numericuid(char *user);
static long	numericgid(char *gp);

void
dirtoxdir(XDir *xd, Dir *d)
{
	//	char buf[NAMELEN+1];
	memset(xd, 0, sizeof *xd);

	xd->name = atom(d->name);
	xd->uid = atom(d->uid);
	xd->gid = atom(d->gid);
	xd->uidno = numericuid(d->uid);
	xd->gidno = numericgid(d->gid);
	xd->mode = d->mode;
	xd->atime = d->atime;
	xd->mtime = d->mtime;
	xd->ctime = 0;
	xd->length = d->length;
	if(xd->mode & DMLINK) {
		xd->mode |= 0777;
		xd->symlink = atom(d->symlink);
	}
};

void
fdtruncate(int fd, vlong size)
{
	ftruncate(fd, size);
	return;
}

static long
numericuid(char *user)
{
	long uid;
	struct passwd *pass;
	static int warned = 0;

	if((uid = altgetnumid(usertab, user)) != ~0)
		return uid;
	else if((pass = getpwnam(user)) != nil)
		return pass->pw_uid;
	if (!warned)
		fprint(2, "warning: getpwnam(3) failed for \"%s\"\n", user);
		warned = 1;
	return 0;
}

static long
numericgid(char *gp)
{
	long gid;
	struct group *gr;
	static int warned = 0;

	if((gid = altgetnumid(grouptab, gp)) != ~0)
		return gid;
	else if((gr = getgrnam(gp)) != nil)
		return gr->gr_gid;
	if (!warned)
		fprint(2, "warning: getgrnam(3) failed for \"%s\"\n", gp);
	warned = 1;
	return 0;
}
